#! /usr/bin/python
# -*- coding: utf-8 -*-
#
# downloader.py
#
# Copyright (C) 2011,2012
# Lorenzo Carbonell Cerezo <lorenzo.carbonell.cerezo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
import sys
import shlex
import subprocess
import os
import comun
from comun import _


def get_formats(url):
	command = '/usr/bin/youtube-dl -F "%s"'%(url)
	result = run(command)
	formats = []
	if result.find('Available formats:')>-1:
		ini = result.find('Available formats:')+19
		sol = result[ini:].replace('\t','')
		for el in sol.split('\n'):
			if el.find(':')>-1:
				formats.append(el.split(':'))
	return formats

def run(command):
	args = shlex.split(command)
	p = subprocess.Popen(args, bufsize=10000, stdout=subprocess.PIPE)
	valor = p.communicate()[0]
	return valor

class DownloadDialog(Gtk.Dialog):

	def __init__(self,url):
		#
		Gtk.Dialog.__init__(self, 'Scope-Youtube | '+_('Download'),None,Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
		self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
		self.set_size_request(500, 140)
		self.connect('close', self.close_application)
		self.set_icon_from_file(comun.ICON)
		#
		self.vbox1 = Gtk.VBox(spacing = 5)
		self.vbox1.set_border_width(5)
		self.get_content_area().add(self.vbox1)
		#
		self.frame1 = Gtk.Frame()
		self.vbox1.add(self.frame1)
		#***************************************************************
		self.vbox2 = Gtk.VBox(spacing = 5)
		self.vbox2.set_border_width(5)
		self.frame1.add(self.vbox2)
		table1 = Gtk.Table(2,2,True)
		table1.set_homogeneous(False)
		self.vbox2.add(table1)
		#
		label1 = Gtk.Label.new(_('Select format')+':')		
		label1.set_alignment(0,0.5)
		table1.attach(label1,0,1,0,1, xoptions = Gtk.AttachOptions.SHRINK, yoptions = Gtk.AttachOptions.SHRINK)		
		liststore = Gtk.ListStore(str, str)
		for aformat in get_formats(url):
			liststore.append([aformat[1],aformat[0]])
		self.entry1 = Gtk.ComboBox.new_with_model(model=liststore)
		renderer_text = Gtk.CellRendererText()
		self.entry1.pack_start(renderer_text, True)
		self.entry1.add_attribute(renderer_text, "text", 0)
		self.entry1.set_active(0)
		table1.attach(self.entry1,1,2,0,1, xoptions = Gtk.AttachOptions.FILL, yoptions = Gtk.AttachOptions.SHRINK)		
		#
		button2 =Gtk.Button(_('Select File'))
		button2.connect('clicked',self.on_button_clicked)
		table1.attach(button2,0,1,1,2, xoptions = Gtk.AttachOptions.SHRINK, yoptions = Gtk.AttachOptions.SHRINK)
		#
		self.entry2 = Gtk.Entry()
		self.entry2.set_editable(False)
		self.entry2.set_width_chars(50)
		filename = os.path.join(os.path.expanduser('~'),url.split('v=')[1])
		filename = filename.split('&')[0]
		self.entry2.set_text(filename)
		table1.attach(self.entry2,1,2,1,2, xoptions = Gtk.AttachOptions.FILL, yoptions = Gtk.AttachOptions.SHRINK)		
		#***************************************************************
		#
		self.show_all()
		#
	def on_button_clicked(self, widget):
		chooser = Gtk.FileChooserDialog(title=('Select where to download the album'),\
										parent=None,action=Gtk.FileChooserAction.SAVE,\
										buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,\
										Gtk.STOCK_OPEN,Gtk.ResponseType.OK))
		chooser.set_filename(self.entry2.get_text())
		if chooser.run() == Gtk.ResponseType.OK:
			filename=chooser.get_filename()
			self.entry2.set_text(filename)
		chooser.destroy()

	def close_application(self, widget, event):
		self.hide()

if __name__ == '__main__':
	if len(sys.argv)>1:
		url = sys.argv[1]		
	else:
		url = 'http://www.youtube.com/watch?v=oVRpfFYRvW4'
	cm = DownloadDialog(url)
	if 	cm.run() == Gtk.ResponseType.ACCEPT:
		tree_iter = cm.entry1.get_active_iter()
		if tree_iter != None:
			model = cm.entry1.get_model()
			dextension = model[tree_iter][0].split('[')[0]
			dformat = model[tree_iter][1]			
			filename = cm.entry2.get_text()+'.'+dextension	
			command = '/usr/bin/youtube-dl "%s" -f %s -o "%s"'%(url,dformat,filename)
			print run(command)
	cm.destroy()
	exit(0)
