#! /usr/bin/python
# -*- coding: utf-8 -*-
#
__author__='atareao'
__date__ ='$26-dic-2011'
#
# Youtube API
#
# Copyright (C) 2011 Lorenzo Carbonell
# lorenzo.carbonell.cerezo@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#

import requests
import json


URL = 'http://gdata.youtube.com/feeds/api/standardfeeds/most_popular?v=2&alt=json'
def read_from_url(url):
	try:
		page = requests.get(url).content
		return json.loads(page.decode('utf-8'))	
	except Exception as e:
		print(e)
	return None


RECORDS_PER_PAGE = 10

def searh_for_videos(search = None, page = 1, categories = [], orderby = 'viewCount', time = 'all_time', rating = 0, hd = False, duration = None, d3 = False):
	'''
	duration:	
		short - Only include videos that are less than four minutes long.
		medium - Only include videos that are between four and 20 minutes long (inclusive).
		long - Only include videos longer than 20 minutes.
	orderby:
		relevance - Entries are ordered by their relevance to a search query. This is the default setting for video search results feeds.
		published - Entries are returned in reverse chronological order. This is the default value for video feeds other than search results feeds.
		viewCount - Entries are ordered from most views to least views.
		rating - Entries are ordered from highest rating to lowest rating.	
	time:
		today (1 day)
		this_week (7 days)
		this_month (1 month)
		all_time
	'''
	if len(categories)>0:
		joined_categories = '/'.join(categories)
		url = 'https://gdata.youtube.com/feeds/api/videos/-/'+joined_categories
		url +='?q=%s&orderby=%s&start-index=%s&max-results=%s&alt=json&v=2'
	else:
		url = 'https://gdata.youtube.com/feeds/api/videos?q=%s&orderby=%s&start-index=%s&max-results=%s&alt=json&v=2'
	if hd:
		url += '&hd=true'
	if d3:
		url += '&3d=true'
	if duration:
		url += '&duration=%s'%duration
	if time:
		url += '&time=%s'%time
	start_index = 1 + (int(page) - 1) * RECORDS_PER_PAGE
	url = url%(search,orderby,start_index,RECORDS_PER_PAGE)
	print('Url: %s'%url)
	ans = read_from_url(url)
	results = []
	if 'feed' in ans.keys():
		if 'entry' in ans['feed'].keys():
			for item in ans['feed']['entry']:
				result = {}
				result['title'] = ''
				result['updated'] = ''
				result['author'] = ''
				result['rating'] = 0
				result['categories'] = ''
				result['license'] = ''
				result['uploaded'] = ''
				result['duration'] = 0
				result['thumbnail'] = []
				result['video'] = []
				result['description'] = ''
				result['viewcount'] = 0
				result['published'] = ''
				result['likes'] = 0
				result['dislikes'] = 0
				result['id'] = ''
				result['toplayer'] = ''
				result['player'] = ''
				result['icon'] = None
				#
				if 'title' in item.keys():
					if '$t' in item['title']:
						result['title'] = item['title']['$t']
				if 'updated' in item.keys():
					if '$t' in item['updated']:
						result['updated'] = item['updated']['$t']			
				if 'author' in item.keys():
					authors = []
					for element in result['author']:
						if 'name' in element:
							if '$t' in element['name']:
								authors.append(element['name']['$t'])
					if len(authors)>0:
						result['author'] = ','.join(authors)
				if 'gd$rating' in item.keys():
					if 'average' in item['gd$rating'].keys():
						result['rating'] = int(float(item['gd$rating']['average']))
				if 'media$group' in item.keys():
					group = item['media$group']
					if 'media$player' in group.keys():
						if 'url' in group['media$player']:
							result['player'] = group['media$player']['url']
					if 'media$category' in group.keys():
						categories = []
						for element in group['media$category']:
							if '$t' in element:
								categories.append(element['$t'])
						if len(categories)>0:
							result['categories'] = ','.join(categories)
					if 'media$license' in group.keys():
						if '$t' in group['media$license']:
							result['license'] = group['media$license']['$t']
					if 'media$thumbnail' in group.keys():
						for element in group['media$thumbnail']:
							if 'url' in element:
								result['thumbnail'].append(element['url'])
						result['icon'] = result['thumbnail'][0]
					if 'media$content' in group.keys():
						for element in group['media$content']:
							if 'url' in element:
								result['video'].append(element['url'])
					if 'media$description' in group.keys():
						if '$t' in group['media$description']:
							result['description'] = group['media$description']['$t']
				if 'yt$uploaded' in item.keys():
					if '$t' in item['yt$uploaded']:
						result['uploaded'] = item['yt$uploaded']['$t']
				if 'yt$duration' in item.keys():
					result['duration'] = item['yt$duration']
				if 'yt$statistics' in item.keys():
					if 'viewCount' in item['yt$statistics']:
						result['viewcount'] = item['yt$statistics']['viewCount']
				if 'published' in item.keys():
					if '$t' in item['published']:
						result['published'] = item['published']['$t']
				if 'yt$rating' in item.keys():
					if 'numLikes' in item['yt$rating']:
						result['likes'] = item['yt$rating']['numLikes']				
				if 'yt$rating' in item.keys():
					if 'numDislikes' in item['yt$rating']:
						result['dislikes'] = item['yt$rating']['numDislikes']
				if 'id' in item.keys():
					if '$t' in item['id']:
						result['id'] = item['id']['$t']
						result['toplayer'] = 'http://www.youtube.com/watch?v='+item['id']['$t'].split(':')[-1]
				if rating == 0 or result['rating']>=rating:
					results.append(result)
					if len(results) >= RECORDS_PER_PAGE:
						return results
	return results

if __name__ == '__main__':
	for result in searh_for_videos('Prueba',page=2,rating=4,categories = ['Tech'],hd=True):
		print result['title'],result['player'],result['author'],result['thumbnail']
	exit(0)
